/*
Im.java
Copyright (C) 2010  Belledonne Communications SARL 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.linphone;

import java.util.Iterator;

import org.linphone.core.LinphoneAddress;
import org.linphone.core.LinphoneCall;
import org.linphone.core.LinphoneCall.State;
import org.linphone.core.LinphoneChatRoom;
import org.linphone.core.LinphoneCore;
import org.linphone.core.LinphoneCore.EcCalibratorStatus;
import org.linphone.core.LinphoneCore.GlobalState;
import org.linphone.core.LinphoneCore.RegistrationState;
import org.linphone.core.LinphoneCoreException;
import org.linphone.core.LinphoneCoreListener;
import org.linphone.core.LinphoneFriend;
import org.linphone.core.LinphoneProxyConfig;
import org.linphone.core.tutorials.TutorialNotifier;


/**
 * Class used to define and to execute the IM background tasks.
 * 
 * Ported from chatroom.c
 *
 * @author Sebastien Dufay, Thibault Jacquemin, Michel Kot, Mathieu Marleix, Vianney Maynadier
 * @author Aubin La, Arnaud Laye, The Huy Phan
 *
 */
public class Im implements LinphoneCoreListener {
	private boolean running;
	private TutorialNotifier notifier;
	private LinphoneChatRoom chatRoom;


	public Im(TutorialNotifier TutorialNotifier) {
		this.notifier = TutorialNotifier;
	}

	public Im() {
		this.notifier = new TutorialNotifier();
	}

	
	public void show(LinphoneCore lc) {}
	public void byeReceived(LinphoneCore lc, String from) {}
	public void authInfoRequested(LinphoneCore lc, String realm, String username) {}
	public void displayStatus(LinphoneCore lc, String message) {}
	public void displayMessage(LinphoneCore lc, String message) {}
	public void displayWarning(LinphoneCore lc, String message) {}
	public void globalState(LinphoneCore lc, GlobalState state, String message) {}
	public void registrationState(LinphoneCore lc, LinphoneProxyConfig cfg,RegistrationState cstate, String smessage) {}
	public void newSubscriptionRequest(LinphoneCore lc, LinphoneFriend lf,String url) {}
	public void notifyPresenceReceived(LinphoneCore lc, LinphoneFriend lf) {}
	public void callState(LinphoneCore lc, LinphoneCall call, State cstate, String msg){}
	public void ecCalibrationStatus(LinphoneCore lc, EcCalibratorStatus status,int delay_ms, Object data) {}
	public void callEncryptionChanged(LinphoneCore lc, LinphoneCall call,boolean encrypted, String authenticationToken) {}
	
	public void textReceived(LinphoneCore lc, LinphoneChatRoom cr,LinphoneAddress from, String message) {
        write("Message ["+message+"] received from ["+from.asString()+"]");
	}
	
	public void launch() throws LinphoneCoreException {
		
		// First instantiate the core Linphone object given only a listener.
		// The listener will react to events in Linphone core.
		
		LinphoneCore lc = LinphoneManager.getLc();

		try {
			// Next step is to create a chat room
			/* Do not create the chat room if the user is not in call */
			while (lc.getRemoteAddress() == null);
			chatRoom = lc.createChatRoom(lc.getRemoteAddress().toString());
			
			/* Write messages that were received while the chat room was not yet created */
			Iterator<String> it = IncallActivity.messageList.iterator();
			while (it.hasNext() == true) {
				String message = it.next();
				receiveText(message);
			}
			IncallActivity.messageList.clear();
			IncallActivity.chatroomIsCreated = true;
			
			// main loop for receiving notifications and doing background linphonecore work
			running = true;
			while (running) {
				lc.iterate();
				try {
					Thread.sleep(50);
				} catch(InterruptedException ie) {
					write("Interrupted!\nAborting");
					return;
				}
			}

		} catch(Exception e) {
			write("You must call someone first.");
		}
	}


	public void stopMainLoop() {
		running=false;
	}

	protected void write(String s) {
		notifier.notify(s);
	}

	protected void sendText(String message) {
		chatRoom.sendMessage(message);
		notifier.notify("You wrote:\n\t" + message);
	}
	
	protected void receiveText(String message) {
		notifier.notify(LinphoneManager.getLc().getRemoteAddress().toString() + " wrote:\n\t" + message);
	}
}
