/*
AboutActivity.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.linphone;

import org.linphone.core.LinphoneCoreException;
import org.linphone.core.tutorials.TutorialNotifier;
import org.shankaa.events.ShankaaIntentListener;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TextView;


/**
 * Activity for displaying and starting the chatroom on Android phone.
 *
 * @author Sebastien Dufay, Thibault Jacquemin, Michel Kot, Mathieu Marleix, Vianney Maynadier
 * @author Aubin La, Arnaud Laye, The Huy Phan
 *
 */
public class ImActivity extends Activity implements ShankaaIntentListener {

	private ScrollView imScroll;
	private TextView imBody;
	private EditText imWrite;
	private Button imSend;
	private static Im im;
	private Handler mHandler =  new Handler();
	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.im);
		
		registerLocalIntents();
		
		imScroll = (ScrollView) findViewById(R.id.im_scroll);
		imBody = (TextView) findViewById(R.id.im_body);
		imWrite = (EditText) findViewById(R.id.im_write);
		imSend = (Button) findViewById(R.id.im_send);
		
		final TutorialNotifier notifier = new AndroidTutorialNotifier(mHandler, imBody);
		
		im = new Im(notifier);
		
		LaunchingThread thread = new LaunchingThread();
		thread.start();
		
		imSend.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if (imWrite.getText().toString().length() > 0) {
					/* Send and display the message */
					try {
						im.sendText(imWrite.getText().toString());
					} catch (Exception e) {
						im.write("You must call someone first");
					}
					scrollDown();
					/* Clear the EditText*/
					imWrite.setText("");
				}
			}
		});
	}
	
	private class LaunchingThread extends Thread {
		@Override
		public void run() {
			super.run();
			try {
				im.launch();
			} catch (LinphoneCoreException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	protected void onDestroy() {
		unregisterLocalIntents();
		super.onDestroy();
	}
	
	@Override
	public void registerLocalIntents() {
		try {
			LinphoneManager manager = LinphoneManager.getInstance();
			manager.registerLocalIntentListener(this);
		} catch (Exception e) {
			
		}
		
	}
	@Override
	public void unregisterLocalIntents() {
		try {
			LinphoneManager manager = LinphoneManager.getInstance();
			manager.unregisterLocalIntentListener(this);
		} catch (Exception e) {
			
		}
		
	}
	
	@Override
	public void onLocalIntent(Intent intent) {
		if (intent.getAction().equals("instantMessageEvent")) {
			try {
				String textMessage = intent.getStringExtra("instantMessage");
				im.receiveText(textMessage);
				scrollDown();
			} catch (Exception e) {
				
			}
		}
	}

	/* Function to be called when a message is sent or received, in order to auto scroll the TextView */
	private void scrollDown() {
	    imScroll.postDelayed(new Runnable() {
	        public void run() {
        		imScroll.pageScroll(ScrollView.FOCUS_DOWN);
        	}
	    }, 100L);
	}
	
	public static Im getIm() {
		return im;
	}
}
